@@ -6,7 +6,7 @@ using namespace Rcpp;
 #ifndef __SURVIVAL_ANALYSIS__
 #define __SURVIVAL_ANALYSIS__
 
-NumericVector fsurvci(double surv, double sesurv, String ct, double z);
+NumericVector fsurvci(double surv, double sesurv, std::string ct, double z);
 
 DataFrame survQuantile(const NumericVector& time, 
                        const IntegerVector& event,
@@ -18,7 +18,9 @@ DataFrame kmest(const DataFrame data,
                 const StringVector& rep,
                 const StringVector& stratum,
                 const std::string time,
+                const std::string time2,
                 const std::string event,
+                const std::string weight,
                 const std::string conftype,
                 const double conflev,
                 const bool keep_censor);
@@ -38,11 +40,12 @@ DataFrame lrtest(const DataFrame data,
                  const StringVector& stratum,
                  const std::string treat,
                  const std::string time,
+                 const std::string time2,
                  const std::string event,
+                 const std::string weight,
                  const double rho1,
                  const double rho2);
 
-
 DataFrame rmest(const DataFrame data,
                 const StringVector& rep,
                 const StringVector& stratum,
@@ -64,7 +67,8 @@ DataFrame rmdiff(const DataFrame data,
                  const bool biascorrection);
 
 struct aftparams {
-  std::string dist;
+  int dist_code; // 1: exponential, 2: weibull, 3: lognormal, 4: normal, 
+                 // 5: loglogistic, 6: logistic
   IntegerVector strata;
   NumericVector tstart;
   NumericVector tstop;
@@ -76,26 +80,22 @@ struct aftparams {
 };
 
 
-double f_llik_1(int p, NumericVector par, void *ex);
+List f_der_1(int p, const NumericVector& par, void *ex);
 
-NumericVector f_score_1(int p, NumericVector par, void *ex);
+NumericMatrix f_ressco_1(int p, const NumericVector& par, void *ex);
 
-NumericMatrix f_info_1(int p, NumericVector par, void *ex);
+NumericMatrix f_jj_1(int p, const NumericVector& par, void *ex);
 
-List f_der_eta_1(NumericVector eta, NumericVector sig, void *ex);
-
-NumericMatrix f_ressco_1(int p, NumericVector par, void *ex);
-
-NumericMatrix f_jj_1(int p, NumericVector par, void *ex);
-
-List liferegloop(int p, NumericVector par, void *ex,
+List liferegloop(int p, const NumericVector& par, void *ex,
                  int maxiter, double eps,
-                 IntegerVector colfit, int ncolfit);
+                 const IntegerVector& colfit, int ncolfit);
 
-double liferegplloop(int p, NumericVector par, void *ex,
+double liferegplloop(int p, const NumericVector& par, void *ex,
                      int maxiter, double eps,
                      int k, int which, double l0);
 
+List f_ld_1(NumericVector eta, NumericVector sig, void *ex);
+
 List liferegcpp(const DataFrame data,
                 const StringVector& rep,
                 const StringVector& stratum,
@@ -107,12 +107,14 @@ List liferegcpp(const DataFrame data,
                 const std::string offset,
                 const std::string id,
                 const std::string dist,
+                const NumericVector& init,
                 const bool robust,
                 const bool plci,
                 const double alpha,
                 const int maxiter,
                 const double eps);
 
+
 NumericMatrix residuals_liferegcpp(const NumericVector& beta,
                                    const NumericMatrix& vbeta,
                                    DataFrame data,
@@ -139,36 +141,27 @@ struct coxparams {
   NumericVector offset;
   NumericMatrix z;
   IntegerVector order1;
-  int method;
+  int method; // 1: breslow, 2: efron
 };
 
-double f_llik_2(int p, NumericVector par, void *ex);
 
-NumericVector f_score_2(int p, NumericVector par, void *ex);
+List f_der_2(int p, const NumericVector& par, void* ex, bool firth);
 
-NumericMatrix f_info_2(int p, NumericVector par, void *ex);
-
-double f_pen_llik_2(int p, NumericVector par, void *ex);
-
-NumericVector f_pen_score_2(int p, NumericVector par, void *ex);
-
-NumericMatrix f_ressco_2(int p, NumericVector par, void *ex);
-
-NumericMatrix f_jj_2(int p, NumericVector par, void *ex);
-
-List phregloop(int p, NumericVector par, void *ex,
+List phregloop(int p, const NumericVector& par, void *ex,
                int maxiter, double eps, bool firth,
-               IntegerVector colfit, int ncolfit);
+               const IntegerVector& colfit, int ncolfit);
 
-double phregplloop(int p, NumericVector par, void *ex,
+double phregplloop(int p, const NumericVector& par, void *ex,
                    int maxiter, double eps, bool firth,
                    int k, int which, double l0);
 
-List f_basehaz(int p, NumericVector par, void *ex);
+List f_basehaz(int p, const NumericVector& par, void *ex);
 
-NumericVector f_resmart(int p, NumericVector par, void *ex);
+NumericMatrix f_ressco_2(int p, const NumericVector& par, void *ex);
 
-List f_ressch(int p, NumericVector par, void *ex);
+NumericVector f_resmart(int p, const NumericVector& par, void *ex);
+
+List f_ressch(int p, const NumericVector& par, void *ex);
 
 List phregcpp(const DataFrame data,
               const StringVector& rep,
@@ -181,6 +174,7 @@ List phregcpp(const DataFrame data,
               const std::string offset,
               const std::string id,
               const std::string ties,
+              const NumericVector& init,
               const bool robust,
               const bool est_basehaz,
               const bool est_resid,
